/* Generates the (alternative) content of a theremin_sintable.c file 
   for the open.theremin.uno project. Compile and upload this sketch 
   to your Arduino and open immediately the serial monitor where the 
   content of the future new theremin_sintable.c file will appear. 
   Once it will be done, click somewhere in the serial monitor window, 
   then press Ctrl-A to select all, and Ctrl-C to copy it. Then move 
   to your second Arduino IDE window and paste everything into an empty 
   theremin_sintable.c file and you are done */

/* Just a simple idea by Theremingenieur Thierry Frenkel 2015 */

/* Everybody is free to use and to modify this code for non-commercial 
   purposes as long as you mention my name and praise my genius! ;-) */

#define tableWidth 1024
#define dacResolutionBits 12

void setup() {
  Serial.begin(57600);
  // Allow the user some time to open the serial monitor
  delay(5000);
  // Print the file header
  Serial.print("/* Theremin WAVE Table - 1024 entries full table, amplitude -2048..2047 */\n\n");
  Serial.print("#include <avr/pgmspace.h>\n\n");
  Serial.print("const int16_t sine_table[1024] PROGMEM = {\\ \n");

  // here the parameters you can play with:
  uint8_t  brightness = 128; //you might put values from 0 to 255
  uint8_t  waveform = 128; //you might put values from 0 to 255

  // some preliminary parameter conditioning, don't touch!
  const float    pi = 3.1415926535;
  float    br = (1.0f + 3.0f * (float)(constrain(brightness, 0, 255) / 255.0f)) * 6.0f / pi;
  float    wf = 0.8f * (float)(constrain(waveform, 0, 255) / 255.0f);
  int16_t  ampl = 1 << (dacResolutionBits - 1);
  int16_t  dacMax = ampl - 1;
  int16_t  dacMin = 0 - ampl;
  float    hp = pi * 2.0f / tableWidth;
  // we need values for intermediate and final results in the function's equation
  int16_t y;

  // now the loop to generate the time steps and function values, and output the latter
  for (int16_t t = 0; t < tableWidth; t++) {
    y = constrain(ampl * tanh((asin(sin(t * hp)) + wf) * br), dacMin, dacMax);
    // formatted output, add a comma after each line but the last
    Serial.print(y, DEC);
    if (t < (tableWidth - 1)) {
      Serial.print(",\n");
    } else {
      Serial.print("\n");
    }

  }
  Serial.print("};\n");
}

void loop() {
  // put your main code here, to run repeatedly:

}
